"""Various kinds of simulated data

This currently only includes a simple linear Gaussian additive MCM method for now.

"""
import numpy as np


class SimulatedData(object):

    def __init__(self, num_samples, num_measured, num_latents, num_diff_FCMs=1, prob_of_FCMs=None):
        self.num_samples = num_samples
        self.num_measured = num_measured
        self.num_latents = num_latents
        self.num_diff_FCMs = num_diff_FCMs
        self.prob_of_FCMs = np.array(prob_of_FCMs)
        self.true_FCMs = make_random_FCMs()
        self.samples = draw_samples()

    def make_random_FCMs(self):
        true_FCMs = np.zeros((num_measured, num_latents, num_diff_FCMs))
        # now randomly make mask
        
        # true_FCMs = dict()
        # for idx in range(self.num_diff_FCMs):
            
        #     true_FCMs['model_' + str(idx)] =
        return true_FCMs

        
    def draw_samples(self):
        pass
    



import numpy as np
# from networkx import from_numpy_matrix, Graph, draw
# from matplotlib.pyplot import show
# from networkx.drawing.nx_pydot import write_dot
# from graphviz import Digraph, render


# probably gets its own method? no, maybe its rather a property of the
# graph object returned by ECC finding? or is ECC finding just a sub
# func in the class/method (ahh, don't know termin) of the UDG, which
# could be a property of the data?
def display_graph(adjacency_matrix):
    graph = from_numpy_matrix(adjacency_matrix, create_using=Graph())
    draw(graph)
    show()

# def display_graph(adjacency_matrix, name):
#     path = '/home/alex_work/Documents/causal_concept_discovery/software/misc/'
#     graph = from_numpy_matrix(adjacency_matrix, create_using=DiGraph())
#     write_dot(graph, path + name + '.gv')
#     graph = Digraph('g', filename=path+name+'.gv')
#     graph.render



from setuptools import setup


setup(name='medil',
      version='0.4.0',
      packages=['medil'],
      install_requires=['numpy'],
      extras_require={
          'dcor': ['dcor'],
          'GAN' : ['pytorch'],
          'vis' : ['matplotlib', 'networkx']
      }
)



import numpy as np
from dcor import pairwise, distance_correlation as distcorr
from multiprocessing import Pool


def dependencies(null_hyp, iota, p_values, alpha):
    null_indep = null_hyp <= iota
    accept_null = p_values >= alpha
    independencies = null_indep & accept_null
    return ~independencies     # dependencies


def hypothesis_test(data, num_resamples, null_corr=None):
    # data must be a matrix of shape num_vars X num_samples
    
    if null_corr is None:
        with Pool() as pool:
            null_corr = pairwise(distcorr, data, pool=pool)
        # null_cov[np.isnan(null_cov)] = 0 # cov with uniform is nan?
        # np.seterr(over='ignore')
        # fixed it---just a problem with the type of my test data

    # initialize aux vars used in loop
    p_values = np.zeros(null_corr.shape)
    num_loops = int(np.ceil(num_resamples / 2))
    for _ in range(num_loops):
        x_1 = permute_within_rows(data)
        x_2 = permute_within_rows(data)

        with Pool() as pool:
            perm_corr = pairwise(distcorr, x_1, x_2, pool=pool)
        p_values += np.array(perm_corr>=null_corr, int)

    # trick for halfing num loops needed for num_resamples because of
    # distcov assymetry; only works if threshold is (nontrivially
    # above) 0
    p_values += p_values.T    
    p_values /= 2 * num_loops

    return p_values, null_corr


def rhoperm(x, name=None, num_perms=1000):
    '''Computes Pearson's correlation coefficient with p-values.

    rho, p = rhoperm(x, num_perms=1000)

    Input variable x has to be of dimension [observations X
    variables]. The p-values are computed by a permutation test.

    '''

    num_samps, num_vars = x.shape

    # Correlation matrix
    rho = np.corrcoef(x, rowvar=False)

    # permutation test for p-values
    p = np.zeros([num_vars, num_vars])
    alt_rho_idx = np.triu_indices(2 * num_vars, num_vars + 1)
    rho_idx = np.triu_indices(num_vars, 1)
    for perm in range(num_perms):
        x_perm = permute_within_columns(x)
        alt_rho = np.corrcoef(x, x_perm, rowvar=False)
        p[rho_idx] += abs(alt_rho[alt_rho_idx]) > abs(rho[rho_idx])
    p = (p + p.T) / num_perms

    if name is not None:
        np.savez(name + '_perm', rho=rho, p=p)
    return rho, p


def permute_within_rows(x):
    # get random new index for col of each element
    col_idx = np.random.sample(x.shape).argsort(axis=1)

    # keep the row index the same
    row_idx = np.tile(np.arange(x.shape[0]), (x.shape[1], 1)).T

    # apply the permutaton matrix to permute x
    return x[row_idx, col_idx]
